<?php
$version = '0.64';
/*
reader class for various c64 fileformats


includes:	cls_DiskImage.php
			cls_LynxImage.php
			cls_T64Image.php
			cls_ArkImage.php
			cls_LbrImage.php
			cls_SpyImage.php
			cls_ArcImage.php
			cls_X64Image.php
			cls_C64Image.php

Supported filetypes: t64, lnx, d64/d71/d81, [psur]*, lbr, spy, x64, c64
'Supported' filetypes: arc(beta...onlyuncompressedorRLE!), ark(alpha...nodataisextractedyet!)

TODO:
	- dNN: skip files which arent real files, filesize should be 0 cause its directing to 18_0 or some shit like that
	- we should really improve our checks, extension isn't the only way to go, filesize and 'headers' should be taken into consideration as well as much as possible
	- more filetypes should be usable: g64 (at least common data extraction like d64), sda, sfx etc.
	- writer-class?! ;-)



PLEASE REPORT ANY BUGS/REVIEWS/SUGGESTIONS TO 3172continuum@gmail.com


feel free to offer your help, send updates to classes


	join our team, let it grow!!!


	MdZ, wreg and ymgve in october 2008


*/


require_once("cls_DiskImage.php");
require_once("cls_LynxImage.php");
require_once("cls_T64Image.php");
require_once("cls_ArkImage.php");
require_once("cls_ArcImage.php");
require_once("cls_LbrImage.php");
require_once("cls_SpyImage.php");
require_once("cls_X64Image.php");
require_once("cls_C64Image.php");

//$html_result .= 'using class_c64reader_v'.$version.' by MdZ, wreg, ymgve';


// usage: var = new c64file(string filename,bool show_dir_only)



//define ('NL', "\n");		//shell output
//define ('NL', "<br>");	//html output
define ('NL', "<br>\n");	//readable html output

function debug_echo($text)
{
//	uncomment this if you want some more debugging output
//	echo $text;
}

class c64file
{
	var $file_types = array("DEL", "SEQ", "PRG", "USR", "REL", "UNK", "UNK", "UNK",
							"UNK", "UNK", "UNK", "UNK", "UNK", "UNK", "UNK", "UNK");



	/* --------------------------------------------------------------------------------
	$directory explained:
	['filename']         : filename
	['filetype']         : filetype (prg, del, seq, rel, usr, unk)
	['filetypeCLOSED']   : filetype pre-evaluated
	['filetypeLOCKED']   : filetype pre-evaluated
	['filetypeSAVE@']    : filetype pre-evaluated
	['filesize']         : actual filesize
	['filesizeblocks']   : filesize in blocks
	['filestart']        : actual start of file in the binary file
	['fileend']          : actual end of file in the binary file
	['filestarttrack']   : track at which the file starts;
	['filestartsector']  : sector at which the file starts;
	['filedata']         : the binary data of the c64 file inside
	['error']            : error
	--------------------------------------------------------------------------------- */

	// Public variables
	var $label = "";
	var $error_in_image = "";
	var $bool_error_in_image = false;
	var $bool_error_inatleastone_file = false;
	var $full_label = "";
	var $disk_id = "";
	var $directory = array();
	var $format_supported = true;


	// Internal variables
	var $raw_data = "";
	var $extract_data = false;
	var $filename = "";

	function c64file($filename, $filedata, $dir_only)
	{
		$this->raw_data = $filedata;
		$this->filename = $filename;
		$this->extract_data = !$dir_only; // for later use without extracting the actual raw data of the file :-)
		if (strlen($filedata) == 0)
		{
			$this->format_supported = false;
		}
		else
		{
			$file_extension = strtolower(substr(strrchr($filename, "."), 1));
			switch($file_extension)
			{
				case "d64" :
				case "d71" :
				case "d81" : $this->read_diskimage($this->raw_data); break;
				case "t64" : $this->read_t64($this->raw_data); break;
				case "lnx" : $this->read_lnx($this->raw_data); break;
				case "ark" : $this->read_ark($this->raw_data); break;
				case "lbr" : $this->read_lbr($this->raw_data); break;
				case "spy" : $this->read_spy($this->raw_data); break;
				case "cbm" : 
				case "prg" : $this->read_prg($this->raw_data); break;
				case "arc" : $this->read_arc($this->raw_data); break;
				case "x64" : $this->read_x64($this->raw_data); break;
				case "c64" : $this->read_c64($this->raw_data); break;

				case preg_match("#^p[0-9]{2}$#", $file_extension) == true : $this->read_pc64($this->raw_data, 'PRG'); break;
				case preg_match("#^s[0-9]{2}$#", $file_extension) == true : $this->read_pc64($this->raw_data, 'SEQ'); break;
				case preg_match("#^u[0-9]{2}$#", $file_extension) == true : $this->read_pc64($this->raw_data, 'USR'); break;
				case preg_match("#^r[0-9]{2}$#", $file_extension) == true : $this->read_pc64($this->raw_data, 'REL'); break;

				default : $this->format_supported = false;
			}
			if ( '' != $this->error_in_image )
				$this->bool_error_in_image = true;

			$dir = $this->directory;
			for ($i=0;$i<sizeOf($dir);$i++)
			{
				if ( '' != $dir[$i]['error'] )
					$this->bool_error_inatleastone_file = true;
			}
		}
	}

	function log_debug($debug, $file = 'debug')
	{
		date_default_timezone_set('Europe/Paris');
		$error = date('d M Y H:i:s',time()) . ' - ' . $debug. NL;
		debug_echo($error);
		$fp = fopen($GLOBALS['config_path_debuglogs'] . '/' . $file . '.txt','a');
		fputs($fp,$error);
		fclose($fp);
	}

	function read_diskimage($c64_image)
	{
		$img = new DiskImage($c64_image);

		$this->label = substr($img->title, 0, 16);
		$this->disk_id = substr($img->title, 18, 5);
		$this->full_label = $img->title;
		$this->error_in_image = $img->error;

		foreach ($img->get_dir() as $entry)
		{
			$file_entry["filetype"] = $this->file_types[$entry["type"] & 15];
			$file_entry["filetypeCLOSED"] = ($entry["type"] & 0x80 ? true : false);
			$file_entry["filetypeLOCKED"] = ($entry["type"] & 0x40 ? true : false);
			$file_entry["filetypeSAVE@"]  = ($entry["type"] & 0x20 ? true : false);

			$file_entry["filestart"] = '';
			$file_entry["fileend"] = '';
			$file_entry["filestarttrack"] = $entry["track"];
			$file_entry["filestartsector"] = $entry["sector"];
			$file_entry["filename"] = $entry["name"];
			$file_entry["filesizeblocksest"] = $entry["size_dir"];

			$ret = $img->get_chain($entry["track"], $entry["sector"]);
			$file_entry["filedata"] = $ret["data"];
			$file_entry["filesize"] = strlen($ret["data"]);
			$file_entry["filesizeblocks"] = ceil($file_entry["filesize"] / 254);
			if ( 0 == $ret["error"] )
			{
				$file_entry["error"] = '';
			}
			else
			{
				$file_entry["error"] = $ret["error"];
			}

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . NL);
		}
	}

	function read_t64($c64_image)
	{
		$img = new T64Image($c64_image);

		$this->label = $img->title;
		$this->disk_id = "";
		$this->error_in_image = $img->error;

		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $this->t64_filetype($entry["disktype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = ceil($entry["filesize"] / 254);
			$file_entry['filestart'] = $entry["offset"];
			$file_entry['fileend'] = $entry["endoffset"];
			$file_entry["filestarttrack"] = '';
			$file_entry["filestartsector"] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . NL);
		}
	}

	function t64_filetype($value)
	{
		switch ($value)
		{
			// the star is a mark for a deleted file 
			case "2"   : $t64type='*PRG' ; break;
			case "129" : $t64type='SEQ'  ; break;
			case "130" : $t64type='PRG'  ; break;
			case "194" : $t64type='PRG<' ; break;
			default    : $t64type='UNK('.$value.')';
		}
		return $t64type;
	}

	function read_pc64($c64_image, $c64_filetype)
	{
		$this->label = "";
		$this->disk_id = "";

		$imagesize = strlen($c64_image);

		$file_entry['filename'] = substr($c64_image, 8, 16);
		$file_entry['filetype'] = $c64_filetype;
		$file_entry['filesize'] = $imagesize - 26;
		$file_entry['filesizeblocks'] = ceil($file_entry['filesize'] / 254);
		$file_entry['filestart'] = 26;
		$file_entry['fileend'] = $imagesize;
		$file_entry["filestarttrack"] = '';
		$file_entry["filestartsector"] = '';
		$file_entry['filedata'] = substr($c64_image, 26);
		$file_entry['error'] = '';
		$this->directory[] = $file_entry;
	}

	function read_prg($c64_image)
	{
		$this->label = "";
		$this->disk_id = "";

		$file_entry['filename'] = basename(substr($this->filename, 0, -4)); // remove .PRG extension
		$file_entry['filetype'] = 'PRG';
		$file_entry['filesize'] = strlen($c64_image);
		$file_entry['filesizeblocks'] = ceil($file_entry['filesize'] / 254);
		$file_entry['filestart'] = 0;
		$file_entry['fileend'] = $file_entry['filesize'];
		$file_entry["filestarttrack"] = '';
		$file_entry["filestartsector"] = '';
		$file_entry['filedata'] = $c64_image;
		$file_entry['error'] = '';
		$this->directory[] = $file_entry;
	}

	function read_lnx($c64_image)
	{
		$img = new LynxImage($c64_image);

		$this->error_in_image = $img->error;
		$this->label = $img->title;
		$this->disk_id = "";

		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $this->lnx_filetype($entry["filetype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = ceil($entry["filesize"] / 254);
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}

	function lnx_filetype($value)
	{
		switch ($value)
		{
			case "P" : $type='PRG'; break;
			case "S" : $type='SEQ'; break;
			case "U" : $type='PRG'; break;
			case "R" : $type='PRG'; break;
			default  : $type='UNK';
		}
		return $type;
	}

	function read_ark($c64_image)
	{
		$img = new ArkImage($c64_image);

		$this->error_in_image = $img->error;
		$this->label = "";
		$this->disk_id = "";
		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $this->t64_filetype($entry["filetype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = ceil($entry["filesize"] / 254);
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}

	function read_lbr($c64_image)
	{
		$img = new LbrImage($c64_image);

		$this->error_in_image = $img->error;
		$this->label = "";
		$this->disk_id = "";
		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $this->t64_filetype($entry["filetype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = ceil($entry["filesize"] / 254);
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}

	function read_spy($c64_image)
	{
		$img = new SpyImage($c64_image);

		$this->error_in_image = $img->error;
		$this->label = "";
		$this->disk_id = "";
		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = t64_filetype($entry["filetype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = $entry["filesizeblocks"];
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}

	function read_arc($c64_image)
	{
		$img = new ArcImage($c64_image);

		$this->error_in_image = $img->error;
		$this->label = "";
		$this->disk_id = "";
		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $this->lnx_filetype($entry["filetype"]);
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = $entry["filesizeblocks"];
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}

	function read_x64($c64_image)
	{
		$img = new X64Image($c64_image);
		debug_echo("<br>len:".strlen($img->imagefile));
		$this->read_diskimage($img->imagefile);
	}

	function read_c64($c64_image)
	{
		$img = new C64Image($c64_image, basename(substr($this->filename, 0, -4)));

		$this->error_in_image = $img->error;
		$this->label = "";
		$this->disk_id = "";
		foreach ($img->entries as $entry)
		{
			$file_entry['filename'] = $entry["filename"];
			$file_entry['filetype'] = $entry["filetype"];
			$file_entry['filesize'] = $entry["filesize"];
			$file_entry['filesizeblocks'] = $entry["filesizeblocks"];
			$file_entry['filestart'] = $entry["start"];
			$file_entry['fileend'] = $entry["end"];
			$file_entry['filestarttrack'] = '';
			$file_entry['filestartsector'] = '';
			$file_entry['filedata'] = $entry["filedata"];
			$file_entry['error'] = $entry["error"];

			$this->directory[] = $file_entry;

			debug_echo("Added " . $file_entry["filename"] . " - " . md5($file_entry["filedata"]) . NL);
		}
	}
}
?>
