<?php
// v0.10
// see class_c64reader.php for more info

class ArcImage
{
	var $imagefile = "";
	var $error = "";
	var $entries = array();

	function ArcImage(&$image)
	{
		$this->imagefile = $image;   
    $offset = 0;
    
    while ($offset<strlen($image))
    {
      if ($image[$offset]=="\x1A"){break;} // end of ARC found!
      
      $u = unpack("Cversion", substr($image, $offset, 1));
      $version         = $u["version"];

      $entry = array();
      $u = unpack("Cmode/vchksum/vorgLOWMED/CorgHIGH/vcompressedblocks/Cfiletype/Clength", substr($image, $offset+1, 10));
      $mode            = $u["mode"];
      $filename_length = $u["length"];

      $entry["filetype"] = substr($image, $offset+9, 1);
      $entry["filesize"] = $u["orgLOWMED"]; // Not always correct!
      $compressed_blocks = $u["compressedblocks"]; 
      $entry["filename"] = substr($image, $offset+11, $filename_length);

      if ($version==2){$offset += 3;}      
      if ($entry["filetype"]=='R')
      {
        $u       = unpack("Crellength/vdate", substr($image, ($offset+12+$u["length"]), 3));
      }
   
      $offset += 10 + $filename_length + 1; // header + filename_length + 1
   
      if ($mode==0)
      {
        // no compression  
        $entry["start"]          = $offset;
        $entry["end"]            = $offset + $entry["filesize"];
        $entry["filesizeblocks"] = ceil($entry["filesize"] / 254); 
        $entry["filedata"]       = substr($image, $entry["start"], $entry["filesize"]);
        $entry["error"]          = ($entry["end"] > strlen($image)) ? 'error' : '';
        $this->error             = $entry["error"] ? 'Error' : '';
        $this->entries[]         = $entry;
      }
      elseif ($mode==1)
      {
        // RLE compression
        $offset++;
        $RLE_byte = $image[$offset-4];  // ''
        $end = $offset + $entry["filesize"];
        $unpacked_image = "";
        while ($offset<$end){  
          if ($image[$offset]==$RLE_byte)
          {
            $u = unpack("Ccount", $image[$offset+1]);
            if ($u["count"]==0){break;} // stop when end-marker is reached '0'  
            $offset += 2;
            for ($j=0;$j<$u["count"];$j++){$unpacked_image .= $image[$offset];}          
          }
          else
          {
           $unpacked_image .= $image[$offset];
          }
          $offset++;
        }

        $entry["start"]    = 0;
        $entry["end"]      = 0;
        $entry["filesize"] = strlen($unpacked_image);
        $entry["filesizeblocks"] = ceil($entry["filesize"] / 254); 
        $entry["filedata"] = $unpacked_image;
        $entry["error"]    = '';
        $this->entries[]   = $entry;
      }
      elseif ($mode==2)
      {
        // $mode==2  Huffman
  			$this->error = "Decompression method not implemented yet!";
        break;
      }
      elseif ($mode==3)
      {
        // $mode==3  LZ (only ARC 2.0)
  			$this->error = "Decompression method not implemented yet!";
      }
      elseif ($mode==4)
      {
        // $mode==4  Huffman with RLE (only ARC 2.0)
  			$this->error = "Decompression method not implemented yet!";
        break;
      }
      elseif ($mode==5)
      {
        // $mode==5  LZ in one pass (only ARC 2.0)
 			  $this->error = "Decompression method not implemented yet!";
        break;
      }
      else 
      {
        // Not a ARC valid compression! 
			 $this->error = "Not a ARC valid compression";
			 return;
      }
      
      $offset += $entry["filesizeblocks"] * 254 - 30 ;
    }    
    
  }
}
?>
